/*
 * Filesystem client functions.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/sys/types.h>

#include <dirent.h>

#include <sys/stat.h>
#include <sys/types.h>

#include <fsclient/fsdesc.h>

EXTERN_C_BEGIN

/* Filesystem mounting. */

long client_mount(file_descriptor_t *desc, const char *devname, const char *mountpath);
file_descriptor_t *client_getfs(file_descriptor_t *desc, const char *name);

/* File and device opening. */

file_descriptor_t *client_open(file_descriptor_t *desc, const char *name, int flags, size_t buffer_size);
file_descriptor_t *client_open_device(file_descriptor_t *desc, const char *cap, int flags, size_t buffer_size);
int client_open_desc(file_descriptor_t *desc, int flags, size_t buffer_size);

/* File and device closing. */

void client_close(file_descriptor_t *desc);
long client_flush(file_descriptor_t *desc);

/* Memory mapped access. */

void *client_mmap(file_descriptor_t *desc, off_t position, size_t length);

/* File reading. */

ssize_t client_read(file_descriptor_t *desc, void *buf, size_t count);
struct dirent *client_readdir(file_descriptor_t *desc);

long client_ensure_read_position(file_descriptor_t *desc, off_t position);

/* File metadata. */

long client_fstat(file_descriptor_t *desc, struct stat *statbuf);

/* File positioning. */

off_t client_seek(file_descriptor_t *desc, off_t offset, int whence);
long client_tell(file_descriptor_t *desc);

/* File notifications. */

long client_begin(file_descriptor_t *desc);
long client_begin_first(file_descriptor_t *desc);
long client_end(file_descriptor_t *desc);
long client_yield(file_descriptor_t *desc);

/* File writing. */

ssize_t client_write(file_descriptor_t *desc, const void *buf, size_t count);

/* Pipe opening. */

long client_pipe(file_descriptor_t *reader, file_descriptor_t *writer);

/* Pipe access. */

void *client_current_region(file_descriptor_t *desc);
void *client_next_region(file_descriptor_t *desc);

EXTERN_C_END
